import datetime
import asyncio
import websockets
import json

messageDict = dict()

# Handler for Server 1
async def handler_server1(websocket):
    print("Client connected to Stream Deck Handler")
    try:
        async for message in websocket:
            print(f"{datetime.datetime.now()}: Stream Deck Handler received message: {message}")
            #await websocket.send(f"Server 1 echo: {message}")

            try:
                messageJSON = json.loads(message)
            except:
                # Not a JSON string just a message
                #print("Getting Text")
                messageDict["text"] = message
                return
            
            try:
                dice = messageJSON["rollType"]
                messageDict["diceroll"] = message
                #print ("Setting dice roll")
            except:
               # print("setting foundry macro")
               # print(json.dumps(messageJSON))

                try:
                    type = messageJSON["type"]
                    if (type == "foundrymacro"):
                        #print(f'Saving Foundry macro: {message}')
                        messageDict["foundrymacro"] = message
                    elif (type == "savageroll"):
                        print(f'Saving savage roll: {message}')
                        messageDict["savageroll"] = message
                except:      
                    print(f"{datetime.datetime.now()}: Unidentified message: {message}")

            #print("Messsage Dict: {messageDict}")

    except websockets.ConnectionClosed as e:
        print(f"Connection closed on Server 1: {e}")

# Handler for Server 2
async def handler_server2(websocket):
    print("Client connected to Client Handler")
    try:
        async for message in websocket:
            print(f"{datetime.datetime.now()}: Client Handler received message: {message}")
            sendMessage = ""
            try:
                if (message == "ROLL"):
                    sendMessage = messageDict["diceroll"]
                elif (message == "FOUNDRYMACRO"):
                   # print(f'Foundry Macro received.')
                    sendMessage = messageDict["foundrymacro"]
                elif (message == "SAVAGEROLL"):
                    print("Got savage roll request")
                    sendMessage = messageDict["savageroll"]
                else:
                    sendMessage = messageDict("text")
                
                print (f"{datetime.datetime.now()}: Client Handler Sent: {sendMessage}")
                await websocket.send(sendMessage)
            except:
                sendMessage = ""
                print("Empty Message Not Sent")


    except websockets.ConnectionClosed as e:
        print(f"Connection closed on Server 2: {e}")

# Start Server 1
async def start_server1():
    server = await websockets.serve(handler_server1, "localhost", 25555)
    print("Server 1 started on ws://localhost:25555")
    await server.wait_closed()

# Start Server 2
async def start_server2():
    server = await websockets.serve(handler_server2, "localhost", 25556)
    print("Server 2 started on ws://localhost:25556")
    await server.wait_closed()

# Main function to run both servers concurrently
async def main():
    # Run both servers concurrently
    await asyncio.gather(
        start_server1(),
        start_server2()
    )

# Run the main event loop
if __name__ == "__main__":
    asyncio.run(main())
